"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._streams = exports.main = void 0;

var _net = _interopRequireDefault(require("net"));

var _fsExtra = _interopRequireDefault(require("fs-extra"));

var _path = _interopRequireWildcard(require("path"));

var _util = _interopRequireDefault(require("util"));

var _crypto = _interopRequireDefault(require("crypto"));

var _debug = _interopRequireDefault(require("debug"));

var _postcss = _interopRequireDefault(require("postcss"));

var _postcssLoadConfig = _interopRequireDefault(require("postcss-load-config"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const debug = (0, _debug.default)('babel-plugin-transform-postcss');
const streams = {
  stderr: process.stderr // overwritable by tests

};
exports._streams = streams;

const md5 = data => _crypto.default.createHash('md5').update(data).digest('hex');

const error = (...args) => {
  let prefix = 'babel-plugin-transform-postcss: ';

  const message = _util.default.format(...args);

  if (streams.stderr.isTTY) {
    prefix = `\x1b[31m${prefix}\x1b[0m`;
  }

  streams.stderr.write(`${prefix}${message}\n`);
};

function outputCSS(_x, _x2) {
  return _outputCSS.apply(this, arguments);
}

function _outputCSS() {
  _outputCSS = _asyncToGenerator(function* (code, file, root = process.cwd(), from, to) {
    if (!from || !to) return;
    let outputPath = to;

    if (!(0, _path.isAbsolute)(outputPath)) {
      outputPath = _path.default.resolve(root, to);
    }

    let sourcePath = from;

    if (!(0, _path.isAbsolute)(sourcePath)) {
      sourcePath = _path.default.resolve(root, from);
    }

    const relativePath = _path.default.relative(sourcePath, file);

    return _fsExtra.default.outputFile(_path.default.resolve(outputPath, relativePath), code);
  });
  return _outputCSS.apply(this, arguments);
}

const main =
/*#__PURE__*/
function () {
  var _main = _asyncToGenerator(function* (socketPath, tmpPath) {
    try {
      _fsExtra.default.mkdirSync(tmpPath);
    } catch (err) {
      if (err.code !== 'EEXIST') {
        throw err;
      }
    }

    const options = {
      allowHalfOpen: true
    };

    const server = _net.default.createServer(options, connection => {
      let data = '';
      connection.on('data', chunk => {
        data += chunk.toString('utf8');
      });
      connection.on('end',
      /*#__PURE__*/
      _asyncToGenerator(function* () {
        try {
          let tokens, cache;
          const {
            cssFile,
            config,
            opts
          } = JSON.parse(data);
          const {
            from,
            to,
            root
          } = opts || {};
          const cachePath = `${_path.default.join(tmpPath, cssFile.replace(/[^a-z]/ig, ''))}.cache`;

          const source = // eslint-disable-next-line no-sync
          _fsExtra.default.readFileSync(cssFile, 'utf8');

          const hash = md5(source); // eslint-disable-next-line no-sync

          try {
            cache = JSON.parse(_fsExtra.default.readFileSync(cachePath, 'utf8'));
          } catch (err) {
            if (err.code !== 'ENOENT') {
              throw err;
            }
          }

          let configPath = _path.default.dirname(cssFile);

          if (config) {
            configPath = _path.default.resolve(config);
          }

          if (cache && cache.hash === hash) {
            connection.end(JSON.stringify(cache.tokens));
            yield outputCSS(cache.code, cssFile, root, from, to);
            return;
          }

          const extractModules = (_, resultTokens) => {
            tokens = resultTokens;
          };

          const {
            plugins,
            options: postcssOpts
          } = yield (0, _postcssLoadConfig.default)({
            extractModules
          }, configPath);
          const runner = (0, _postcss.default)(plugins);
          const code = yield runner.process(source, Object.assign({
            from: cssFile,
            to: cssFile // eslint-disable-line id-length

          }, postcssOpts)); // output css files

          yield outputCSS(code.css, cssFile, root, from, to);
          cache = {
            hash,
            tokens,
            code: code.css // eslint-disable-next-line no-sync

          };

          _fsExtra.default.writeFileSync(cachePath, JSON.stringify(cache));

          connection.end(JSON.stringify(tokens));
        } catch (err) {
          error(err.stack);
          connection.end();
        }
      }));
    });

    if (_fsExtra.default.existsSync(socketPath)) {
      error(`Server already running on socket ${socketPath}`);
      process.exit(1);
    } else {
      yield new Promise((resolve, reject) => {
        server.on('error', err => reject(err));
        server.on('listening', () => {
          const handler = () => {
            _fsExtra.default.unlinkSync(socketPath); // eslint-disable-line no-sync

          };

          server.on('close', () => {
            process.removeListener('exit', handler);
            process.removeListener('SIGINT', handler);
            process.removeListener('SIGTERM', handler);
          });
          process.on('exit', handler);
          process.on('SIGINT', handler);
          process.on('SIGTERM', handler);
          resolve();
        });
        server.listen(socketPath, () => {
          debug(`babel-plugin-transform-postcss server running on socket ${socketPath}`);
        });
      });
      return server;
    }
  });

  function main(_x3, _x4) {
    return _main.apply(this, arguments);
  }

  return main;
}();
/* istanbul ignore if */


exports.main = main;

if (require.main === module) {
  _asyncToGenerator(function* () {
    try {
      yield main(...process.argv.slice(2));
    } catch (err) {
      process.stderr.write(`${err.stack}\n`);
      process.exit(1);
    }
  })();
}
//# sourceMappingURL=postcss-server.js.map